------------------------------------------------------------------------
-- @class Branch
-- Class representing a player's branch
-- Root of the player, can grow or or decay or do whatever branches do
------------------------------------------------------------------------

class "Root"{
}

function Root:__init__(p_parent, p_hook)
    print("new root")
    self.parent     =   p_parent
    self.species    =   p_parent.species
    self.image      =   self.species.root_image
    self.anchor     =   {}
    self.hook       =   {}
    self.center     =   {}
    self.roots      =   {}
    self.level      =   0
    self.angle      =   0
    self.time       =   0
    self.selected   =   false
    self.hook.x, self.hook.y        =   unpack(self.species.root_hook)
    self.anchor.x, self.anchor.y    =   unpack(self.species.root_anchor)
    self.x_pos, self.y_pos          =   0,0
    if p_hook then
        self.x_pos, self.y_pos      =   p_hook.x, p_hook.y --if several hooks, get the good one
    else
        self.x_pos, self.y_pos      =   p_parent.hook.x, p_parent.hook.y
    end
end

function Root:setTransparent(p_bool)
    self.transparent    =   p_bool

end

function Root:update(dt)
    self.time   =   self.time + dt
end

function Root:levelUp()
    self.level  =   self.level + 1
    self.parent:levelUp()
end

function Root:confirmRoot(p_ghost)
    local r = Root(self)
    r:fromGhost(p_ghost)
    table.insert(self.roots, r)
    table.insert(self:getTree().roots, 0, r)
    --~ table.insert(GAME_OBJECTS,1, r)
    self:levelUp()
    print("GameObject has now: "..#GAME_OBJECTS)
end

function Root:draw()
    if self.selected then
        love.graphics.setColor(255, 100, 100)
    else
        love.graphics.setColor(255, 255, 255)
    end
    love.graphics.draw(self:getDrawable())

    if DEBUG then
        love.graphics.setColor(255,0,0)
        love.graphics.circle("fill",self.anchor.x, self.anchor.y, 10)
        love.graphics.setColor(0,255,0)
        love.graphics.circle("fill",self.x_pos, self.y_pos, 10)
        love.graphics.setColor(0,0,255)
        love.graphics.circle("fill", self.hook.x, self.hook.y, 10)
        love.graphics.setColor(255,255,255)
        love.graphics.circle("fill", self.center.x, self.center.y, 15)
    end
end

function Root:fromGhost(p_ghost)
    self.anchor =   p_ghost.anchor
    self.x_pos  =   p_ghost.x_pos
    self.y_pos  =   p_ghost.y_pos
    self.angle  =   p_ghost.angle
    local v_hook  =   vector(self.hook.x - self.anchor.x, self.hook.y - self.anchor.y)
    v_hook:rotate_inplace(self.angle)
    self.hook.x, self.hook.y        =   v_hook.x+self.x_pos, v_hook.y+self.y_pos
    self.center.x, self.center.y    =   v_hook.x/2+self.x_pos, v_hook.y/2+self.y_pos

end

function Root:getDrawable(p_image)
    local scale =   1 + (self.level/self.species.root_level_factor)
    if not p_image then

        return self.image, self.x_pos, self.y_pos, self.angle, scale, 1, self.anchor.x, self.anchor.y
    else
        return p_image, self.x_pos, self.y_pos, self.angle, scale, 1, self.anchor.x, self.anchor.y
    end

end

function Root:getTree()
    return self.parent:getTree()
end

function Root:appendRoot()
    local t = self:getTree()
    local b = GhostRoot(self)

    t.appending_element =   b
    t.appending         =   true

    --Correct this

    print("Seed is inserting the dummy in GUI")
    table.insert(GAME_OBJECTS, b)
    print("Gui has now: "..#GUI)
    b=nil


end

function Root:getActions()
    if #self.roots >= self.species.max_roots then
        return {"Decay"}
    else
        return {"Root","Decay"}
    end
end

function Root:testPoint(p_x, p_y)
    print("Testing point on root with angle: "..self.angle)
    local test = point_vs_raabb (p_x, p_y, self.center.x, self.center.y , self.image:getWidth()/2, self.image:getHeight()/2, math.pi*2 - self.angle)
    return test


end

