------------------------------------------------------------------------
-- @class Tree
-- Class representing a player's treen
-- Contains branches and roots. Can append/decay them and make fruits
------------------------------------------------------------------------

class "Tree"{
}

function Tree:__init__(p_species)
    self.core           =   {}
    self.branches       =   {}
    self.roots          =   {}
    self.species        =   p_species
    self.branch_image   =   p_species.branch_image
    self.root_image     =   p_species.root_image
    self.transparency   =   true
    self.appending      =   false
    print("New player tree: "..self.species.name)
end

function Tree:draw()
    --Draw my branches
    for i, b in pairs(self.branches) do
        b:draw()
    end

    --Draw my roots
    for i, r in pairs(self.roots) do
        r:draw()
    end

    --Draw my seed
    if self.core.draw then
        self.core:draw()
    end
end

function Tree:newBranch(p_parent)
    new_branch  =   Branch(p_parent)
    table.insert(self.branches, new_branch)
    print("Branch: "..new_branch.x_pos.." "..new_branch.y_pos)
    return new_branch

end

function Tree:newRoot(p_root)
    new_root    =   Root(p_root)
    table.insert(self.roots, new_root)
    print("New root added to tree")
    return new_root

end

function Tree:setTransparent(p_bool)
    print("Setting transparency on tree")
    print(p_bool)
    self.transparent    =   p_bool
    for idx,branch in pairs(self.branches) do
        branch:setTransparent(p_bool)
    end

end

function Tree:setWorld(p_world)
    self.world  =   p_world

end

function Tree:setCamera(p_cam)
    self.camera =   p_cam

end


function Tree:getTree()
    return self
end

function Tree:setCore(seed)
    seed.parent =   self
end

function Tree:testPoint(p_x, p_y)

    if seed:testPoint(p_x, p_y) then
        return seed
    end

    for id, branch in pairs(self.branches) do
        if branch:testPoint(p_x, p_y) then
            return branch
        end
    end

    for id, root in pairs(self.roots) do
        if root:testPoint(p_x, p_y) then
            return root
        end
    end
end

function Tree:update(dt)
    --Update my branches
    for i, b in pairs(self.branches) do
        b:update(dt)
    end

    --Update my roots
    for i, r in pairs(self.roots) do
        r:update(dt)
    end

    --Update my seed
    if self.core.update then
        self.core:update(dt)
    end
end
